@@ -1,3 +1,4 @@
+import builtins
 import datetime
 import json
 import pathlib
@@ -8,23 +9,59 @@ import webbrowser
 import requests
 from bs4 import BeautifulSoup as Soup
 
+T = typing.TypeVar("T")
+
 try:
-    import colorama as colour
+    from rich import print, progress
 except ImportError:
     RED = ""
     YELLOW = ""
     GREEN = ""
     BLUE = ""
+    GOLD = ""
     RESET = ""
-else:
-    colour.init()
-    RED = colour.Fore.LIGHTRED_EX
-    YELLOW = colour.Fore.LIGHTYELLOW_EX
-    GREEN = colour.Fore.LIGHTGREEN_EX
-    BLUE = colour.Fore.LIGHTBLUE_EX
-    RESET = colour.Fore.RESET
 
-from .data import DATA_DIR, DEFAULT_YEAR, URL, WAIT_TIME, get_cookie
+    def wait(msg: str, secs: float) -> None:
+        print(msg)
+        time.sleep(secs)
+
+    def work(msg: str, worker: typing.Callable[[], T]) -> T:
+        print(msg)
+        return worker()
+
+else:
+    RED = "[red]"
+    YELLOW = "[yellow]"
+    GREEN = "[green]"
+    BLUE = "[blue]"
+    GOLD = "[gold1]"
+    RESET = "[/]"
+
+    def wait(msg: str, secs: float) -> None:
+        for _ in progress.track(
+            builtins.range(int(10 * secs)),
+            description=msg,
+            show_speed=False,
+            transient=True,
+        ):
+            time.sleep(0.1)
+
+    def _rich_work(msg: str, worker: typing.Callable[[], T]) -> T:
+        with progress.Progress(
+            progress.TextColumn("{task.description}"),
+            progress.SpinnerColumn(),
+            progress.TimeElapsedColumn(),
+            transient=True,
+        ) as bar:
+            task = bar.add_task(msg)
+            val = worker()
+            bar.advance(task)
+            return val
+
+    work = _rich_work
+
+
+from .data import DATA_DIR, DEFAULT_YEAR, RANK, TODAY, URL, WAIT_TIME, get_cookie
 
 
 def _open_page(page: str) -> None:
@@ -47,13 +84,13 @@ def _pretty_print(message: str) -> None:
         print(YELLOW + message + RESET)
     elif message.startswith("That's not"):
         print(RED + message + RESET)
+    elif message.startswith("You got rank"):
+        print(GOLD + message + RESET)
     else:
         raise ValueError("Failed to parse response.")
 
 
-def fetch(
-    day: int, year: int = DEFAULT_YEAR
-) -> str:  # Might consider a default TODAY for day
+def fetch(day: int = TODAY, year: int = DEFAULT_YEAR) -> str:
     """Fetch and return the input for `day` of `year`.
 
     All inputs are cached in `aoc_helper.DATA_DIR`."""
@@ -69,20 +106,25 @@ def fetch(
         unlock = datetime.datetime(year, 12, day, 5)
         now = datetime.datetime.utcnow()
         if now < unlock:
+            # On the first day, run a stray request to validate the user's token
             if day == 1:
                 resp = requests.get(
                     URL.format(day=1, year=2015) + "/input", cookies=get_cookie()
                 )
                 if resp.status_code == 400:
                     token_file = DATA_DIR / "token.txt"
-                    token = input(
-                        "Your token has expired. Please enter your new token\n>>> "
+                    print(
+                        f"{RED}Your token has expired. Please enter your new"
+                        f" token.{RESET}"
                     )
+                    token = input(">>> ")
                     token_file.write_text(token)
                     return fetch(day, year)
                 now = datetime.datetime.utcnow()
-            print(YELLOW + "Waiting for puzzle unlock..." + RESET)
-            time.sleep((unlock - now).total_seconds())
+            wait(
+                f"{YELLOW}Waiting for puzzle unlock...{RESET}",
+                (unlock - now).total_seconds(),
+            )
             print(GREEN + "Fetching input!" + RESET)
             _open_page(URL.format(day=day, year=year))
         resp = requests.get(
@@ -91,15 +133,13 @@ def fetch(
         if not resp.ok:
             if resp.status_code == 400:
                 token_file = DATA_DIR / "token.txt"
-                token = input(
-                    "Your token has expired. Please enter your new token\n>>> "
+                print(
+                    f"{RED}Your token has expired. Please enter your new token.{RESET}"
                 )
+                token = input(">>> ")
                 token_file.write_text(token)
                 return fetch(day, year)
             raise ValueError("Received bad response")
-        # Note to star: May consider rstrip instead -- I don't know if AoC will ever
-        # publish input that has significant whitespace at the beginning though. --
-        # should now be fixed: stripping newlines only
         data = resp.text.strip("\n")
         input_path.write_text(data)
         return data
@@ -134,19 +174,17 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
         print(
             f"Day {BLUE}{day}{RESET} part {BLUE}{part}{RESET} "
             "has already been solved.\nThe solution was: "
-            f"{BLUE}{solution}{RESET}"  # "\nResponse was:\n"
+            f"{BLUE}{solution}{RESET}"
         )
-        return  # _pretty_print(solutions[part_][solution])
-        # printing the response here is pretty pointless, the user
-        # already knows it's correct
-
-        # salt: Yeah, but AoC will sometimes tell you what you placed (if you're in the top 1000) and you might want to see it!
+        if match := RANK.search(solutions[part_][solution]):
+            _pretty_print(match.group(0))
+        return
 
     # Check if answer has already been submitted
     if answer_ in solutions[part_]:
         print(
-            f"{YELLOW}Solution: {BLUE}{answer}{YELLOW} to part "
-            f"{BLUE}{part}{YELLOW} has already been submitted.\n"
+            f"{YELLOW}Solution: {BLUE}{answer}{RESET} to part "
+            f"{BLUE}{part}{RESET} has already been submitted.\n"
             f"Response was:{RESET}"
         )
         return _pretty_print(solutions[part_][answer_])
@@ -171,14 +209,19 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
                 return submit(day, part, answer, year)
             raise ValueError("Received bad response")
 
-        msg: str = Soup(resp.text, "html.parser").article.text
+        article = Soup(resp.text, "html.parser").article
+        assert article is not None
+        msg = article.text
 
         if msg.startswith("You gave"):
             print(RED + msg + RESET)
             wait_match = WAIT_TIME.search(msg)
+            assert wait_match is not None
             pause = 60 * int(wait_match[1] or 0) + int(wait_match[2])
-            print(f"{YELLOW}Waiting {BLUE}{pause}{YELLOW} seconds to retry...{RESET}")
-            time.sleep(pause)
+            wait(
+                f"{YELLOW}Waiting {BLUE}{pause}{RESET} seconds to retry...{RESET}",
+                pause,
+            )
         else:
             break
     _pretty_print(msg)
@@ -196,6 +239,25 @@ def submit(day: int, part: int, answer: typing.Any, year: int = DEFAULT_YEAR) ->
         json.dump(solutions, f)
 
 
+def submit_25(year: str):
+    print(f"{GREEN}Finishing Advent of Code {BLUE}{year}{RESET}!{RESET}")
+    resp = requests.post(
+        url=URL.format(day="25", year=year) + "/answer",
+        cookies=get_cookie(),
+        data={"level": "2", "answer": "0"},
+    )
+    if not resp.ok:
+        if resp.status_code == 400:
+            token_file = DATA_DIR / "token.txt"
+            token = input("Your token has expired. Please enter your new token\n>>> ")
+            token_file.write_text(token)
+            return submit_25(year)
+        raise ValueError("Received bad response")
+
+    print("Response from the server:")
+    print(resp.text.strip())
+
+
 def lazy_submit(
     day: int, solution: typing.Callable[[], typing.Any], year: int = DEFAULT_YEAR
 ) -> None:
@@ -204,6 +266,32 @@ def lazy_submit(
     solution is expected to be named 'part_one' or 'part_two'
     """
     part = 1 if solution.__name__ == "part_one" else 2
-    if not (DATA_DIR / str(year) / str(day) / f"{part}.solution").exists():
-        if (answer := solution()) is not None:
-            submit(day, part, answer, year)
+    submission_dir = DATA_DIR / str(year) / str(day)
+    if day == 25 and part == 2:
+        # Don't try to submit part 2 if part 1 isn't solved
+        if (submission_dir / "1.solution").exists():
+            submit_25(str(year))
+    solution_file = submission_dir / f"{part}.solution"
+    # Check if solved
+    if solution_file.exists():
+        # Load cached solutions
+        submissions = submission_dir / "submissions.json"
+        with submissions.open() as f:
+            solutions = json.load(f)
+
+        solution_ = solution_file.read_text()
+        print(
+            f"Day {BLUE}{day}{RESET} part {BLUE}{part}{RESET} "
+            "has already been solved.\nThe solution was: "
+            f"{BLUE}{solution_}{RESET}"
+        )
+        if match := RANK.search(solutions[str(part)][solution_]):
+            _pretty_print(match.group(0))
+    elif (
+        answer := work(
+            f"{YELLOW}Running part"
+            f" {RESET}{BLUE}{part}{RESET}{YELLOW} solution...{RESET}",
+            solution,
+        )
+    ) is not None:
+        submit(day, part, answer, year)
